
#include <stdio.h>
#include <stdlib.h>
#include <GL/glew.h>
#include <GLFW/glfw3.h>
#include <iostream>
#include <cassert>
#include <cmath>
#include <stdio.h>
#include "GLSL.h"
#include "RenderingHelper.h"
#include "tiny_obj_loader.h"
#include "glm/glm.hpp"
#include "glm/gtc/matrix_transform.hpp" //perspective, trans etc
#include "glm/gtc/type_ptr.hpp" //value_ptr

#define SPEED 0.05;
#define LEFT 90;
#define RIGHT -90;

GLFWwindow* window;
using namespace std;

vector<tinyobj::shape_t> dog;
vector<tinyobj::shape_t> cube;
vector<tinyobj::material_t> materials;

const double pi = 3.1415926535897;

int walls[4][10];
int g_norm = 0;
int g_SM = 0;
int g_width;
int g_height;
int nIndices;
int leftRotate = 0;
int rightRotate = 0;
int wallIndex = 0;
int zIndex = 0;
int material = 0;
float g_Camtrans = -2.5;
float g_xangle = 0;
float g_angle = 0;
float phi = 0;
float theta = -pi / 2;
float time = 0;
float jumpTime = 0;
float maxHeight = 3;
float addSpeed = 0;
bool armFlag;
bool jumpReady = true;
bool gameStart = false;
bool falling = false;
int g_mat_id = 0;
glm::vec3 g_trans(0, 0, 0);
glm::vec3 g_light(0, 0, 0);

glm::vec3 eye(0, 0, 6);
glm::vec3 center(0, 0, -1);

RenderingHelper ModelTrans;

GLuint ShadeProg;
GLuint posBufObjB = 0;
GLuint norBufObjB = 0;
GLuint indBufObjB = 0;

GLuint posBufObjC = 0;
GLuint norBufObjC = 0;
GLuint indBufObjC = 0;

GLuint posBufObjG = 0;
GLuint norBufObjG = 0;
//Handles to the shader data
GLint h_aPosition;
GLint h_aNormal;
GLint h_uModelMatrix;
GLint h_uViewMatrix;
GLint h_uProjMatrix;
GLint h_uLightPos;
GLint h_uMatAmb, h_uMatDif, h_uMatSpec, h_uMatShine;
GLint h_uShadeM;
GLint uNormal;

/* helper function to make sure your matrix handle is correct */
inline void safe_glUniformMatrix4fv(const GLint handle, const GLfloat data[]) {
  if (handle >= 0)
    glUniformMatrix4fv(handle, 1, GL_FALSE, data);
}

/* helper function to send materials to the shader - you must create your own */
void SetMaterial(int i) {
 
  glUseProgram(ShadeProg);
  switch (i) {
    case 0: //shiny blue plastic
        glUniform3f(h_uMatAmb, 0.02, 0.02, 0.1);
        glUniform3f(h_uMatDif, 0.0, 0.08, 0.5);
        glUniform3f(h_uMatSpec, 0.14, 0.14, 0.4);
        glUniform1f(h_uMatShine, 120.0);
        break;
    case 1: // flat grey
        glUniform3f(h_uMatAmb, 0.13, 0.13, 0.14);
        glUniform3f(h_uMatDif, 0.3, 0.3, 0.4);
        glUniform3f(h_uMatSpec, 0.3, 0.3, 0.4);
        glUniform1f(h_uMatShine, 4.0);
        break;
    case 2: //gold
        glUniform3f(h_uMatAmb, 0.09, 0.07, 0.08);
        glUniform3f(h_uMatDif, 0.91, 0.782, 0.82);
        glUniform3f(h_uMatSpec, 1.0, 0.913, 0.8);
        glUniform1f(h_uMatShine, 200.0);
        break;
    case 3: //mint
        glUniform3f(h_uMatAmb, 0.09, 0.07, 0.08);
        glUniform3f(h_uMatDif, 0.3, 0.809, 0.809);
        glUniform3f(h_uMatSpec, 1.0, 0.913, 0.8);
        glUniform1f(h_uMatShine, 100.0);
        break;
    case 4: //ground
        glUniform3f(h_uMatAmb, 0.0, 0.0, 0.0);
        glUniform3f(h_uMatDif, 0.3, 0.0, 0.809);
        glUniform3f(h_uMatSpec, 0.2, 0.2, 0.2);
        glUniform1f(h_uMatShine, 1.0);
        break;
	case 5: //ground
        glUniform3f(h_uMatAmb, 0.0, 0.0, 0.0);
        glUniform3f(h_uMatDif, 0.1, 0.5, 0.809);
        glUniform3f(h_uMatSpec, 0.2, 0.2, 0.2);
        glUniform1f(h_uMatShine, 1.0);
        break;
	case 6: //ground
        glUniform3f(h_uMatAmb, 0.0, 0.0, 0.0);
        glUniform3f(h_uMatDif, 0.2, 0.0, 0.8);
        glUniform3f(h_uMatSpec, 0.2, 0.2, 0.2);
        glUniform1f(h_uMatShine, 1.0);
        break;
  }
}

/* helper function to set projection matrix - don't touch */
void SetProjectionMatrix() {
  glm::mat4 Projection = glm::perspective(45.0f, (float)g_width/g_height, 0.1f, 100.f);
  safe_glUniformMatrix4fv(h_uProjMatrix, glm::value_ptr(Projection));
}

/* camera controls - do not change beyond the current set up to rotate*/
void SetView() {
  glm::mat4 Trans = glm::lookAt(
    eye,
    center,
    vec3(0, 1, 0)
  );
  safe_glUniformMatrix4fv(h_uViewMatrix, glm::value_ptr(Trans));
}

/* model transforms */
void SetModel(vec3 trans, float rot, float sc, int axis) {
	glm::mat4 Trans = glm::translate(glm::mat4(1.0f), trans);
	glm::mat4 Rotate, GRotate;
	if (axis == 0) {//rotate around x-axis
		Rotate = glm::rotate(glm::mat4(1.0f), rot, glm::vec3(1.0f, 0, 0));
	}
	else if (axis == 1) {//rotate around y-axis
		Rotate = glm::rotate(glm::mat4(1.0f), rot, glm::vec3(0.0f, 1, 0));
	}
	else if (axis == 2) {//rotate around z-axis
		Rotate = glm::rotate(glm::mat4(1.0f), rot, glm::vec3(0.0f, 0, 1));
	}
	if (axis != 1) {
		GRotate = glm::rotate(glm::mat4(1.0f), (float)leftRotate + rightRotate, glm::vec3(0.0f, 0, 1));
	}
	else {
		GRotate = glm::rotate(glm::mat4(1.0f), (float)0.0, glm::vec3(0.0f, 0, 1));
	}
	glm::mat4 Sc = glm::scale(glm::mat4(1.0f), vec3(sc));
	glm::mat4 com = GRotate*Trans*Rotate*Sc;
	safe_glUniformMatrix4fv(h_uModelMatrix, glm::value_ptr(com));
}

//Given a vector of shapes which has already been read from an obj file
// resize all vertices to the range [-1, 1]
void resize_obj(std::vector<tinyobj::shape_t> &shapes){
    float minX, minY, minZ;
    float maxX, maxY, maxZ;
    float scaleX, scaleY, scaleZ;
    float shiftX, shiftY, shiftZ;
    float epsilon = 0.001;

    minX = minY = minZ = 1.1754E+38F;
    maxX = maxY = maxZ = -1.1754E+38F;

    //Go through all vertices to determine min and max of each dimension
    for (size_t i = 0; i < shapes.size(); i++) {
        for (size_t v = 0; v < shapes[i].mesh.positions.size() / 3; v++) {
            if(shapes[i].mesh.positions[3*v+0] < minX) minX = shapes[i].mesh.positions[3*v+0];
            if(shapes[i].mesh.positions[3*v+0] > maxX) maxX = shapes[i].mesh.positions[3*v+0];

            if(shapes[i].mesh.positions[3*v+1] < minY) minY = shapes[i].mesh.positions[3*v+1];
            if(shapes[i].mesh.positions[3*v+1] > maxY) maxY = shapes[i].mesh.positions[3*v+1];

            if(shapes[i].mesh.positions[3*v+2] < minZ) minZ = shapes[i].mesh.positions[3*v+2];
            if(shapes[i].mesh.positions[3*v+2] > maxZ) maxZ = shapes[i].mesh.positions[3*v+2];
        }
    }
   //From min and max compute necessary scale and shift for each dimension
   float maxExtent, xExtent, yExtent, zExtent;
   xExtent = maxX-minX;
   yExtent = maxY-minY;
   zExtent = maxZ-minZ;
   if (xExtent >= yExtent && xExtent >= zExtent) {
     maxExtent = xExtent;
   }
   if (yExtent >= xExtent && yExtent >= zExtent) {
     maxExtent = yExtent;
   }
   if (zExtent >= xExtent && zExtent >= yExtent) {
     maxExtent = zExtent;
   }
    scaleX = 2.0 /maxExtent;
    shiftX = minX + (xExtent/ 2.0);
    scaleY = 2.0 / maxExtent;
    shiftY = minY + (yExtent / 2.0);
    scaleZ = 2.0/ maxExtent;
    shiftZ = minZ + (zExtent)/2.0;

    //Go through all verticies shift and scale them
    for (size_t i = 0; i < shapes.size(); i++) {
        for (size_t v = 0; v < shapes[i].mesh.positions.size() / 3; v++) {
            shapes[i].mesh.positions[3*v+0] = (shapes[i].mesh.positions[3*v+0] - shiftX) * scaleX;
            assert(shapes[i].mesh.positions[3*v+0] >= -1.0 - epsilon);
            assert(shapes[i].mesh.positions[3*v+0] <= 1.0 + epsilon);
            shapes[i].mesh.positions[3*v+1] = (shapes[i].mesh.positions[3*v+1] - shiftY) * scaleY;
            assert(shapes[i].mesh.positions[3*v+1] >= -1.0 - epsilon);
            assert(shapes[i].mesh.positions[3*v+1] <= 1.0 + epsilon);
            shapes[i].mesh.positions[3*v+2] = (shapes[i].mesh.positions[3*v+2] - shiftZ) * scaleZ;
            assert(shapes[i].mesh.positions[3*v+2] >= -1.0 - epsilon);
            assert(shapes[i].mesh.positions[3*v+2] <= 1.0 + epsilon);
        }
    }
}

void loadShapes(const string &objFile, std::vector<tinyobj::shape_t>& shapes)
{
	string err = tinyobj::LoadObj(shapes, materials, objFile.c_str());
	if(!err.empty()) {
		cerr << err << endl;
	}
   resize_obj(shapes);
}

void initModel(std::vector<tinyobj::shape_t>& shape) {

	// Send the position array to the GPU
	const vector<float> &posBuf = shape[0].mesh.positions;
	glGenBuffers(1, &posBufObjB);
	glBindBuffer(GL_ARRAY_BUFFER, posBufObjB);
	glBufferData(GL_ARRAY_BUFFER, posBuf.size()*sizeof(float), &posBuf[0], GL_STATIC_DRAW);
	
	// Send the normal array to the GPU
	vector<float> norBuf;
	glm::vec3 v1, v2, v3;
	glm::vec3 edge1, edge2, norm;
	int idx1, idx2, idx3;
   //for every vertex initialize the vertex normal to 0
   for (int j = 0; j < shape[0].mesh.positions.size()/3; j++) {
      norBuf.push_back(0);
      norBuf.push_back(0);
      norBuf.push_back(0);
   }
   //process the mesh and compute the normals - for every face
   //add its normal to its associated vertex
   for (int i = 0; i < shape[0].mesh.indices.size()/3; i++) {
      idx1 = shape[0].mesh.indices[3*i+0];
      idx2 = shape[0].mesh.indices[3*i+1];
      idx3 = shape[0].mesh.indices[3*i+2];
      v1 = glm::vec3(shape[0].mesh.positions[3*idx1 +0],shape[0].mesh.positions[3*idx1 +1], shape[0].mesh.positions[3*idx1 +2]);
      v2 = glm::vec3(shape[0].mesh.positions[3*idx2 +0],shape[0].mesh.positions[3*idx2 +1], shape[0].mesh.positions[3*idx2 +2]);
      v3 = glm::vec3(shape[0].mesh.positions[3*idx3 +0],shape[0].mesh.positions[3*idx3 +1], shape[0].mesh.positions[3*idx3 +2]);
      edge1 = v2 - v1;
      edge2 = v3 - v1;
      norm = glm::cross(edge1, edge2);
      norBuf[3*idx1+0] += (norm.x);
      norBuf[3*idx1+1] += (norm.y);
      norBuf[3*idx1+2] += (norm.z);
      norBuf[3*idx2+0] += (norm.x);
      norBuf[3*idx2+1] += (norm.y);
      norBuf[3*idx2+2] += (norm.z);
      norBuf[3*idx3+0] += (norm.x);
      norBuf[3*idx3+1] += (norm.y);
      norBuf[3*idx3+2] += (norm.z);
   }
	glGenBuffers(1, &norBufObjB);
	glBindBuffer(GL_ARRAY_BUFFER, norBufObjB);
	glBufferData(GL_ARRAY_BUFFER, norBuf.size()*sizeof(float), &norBuf[0], GL_STATIC_DRAW);
	
	// Send the index array to the GPU
	const vector<unsigned int> &indBuf = shape[0].mesh.indices;
	glGenBuffers(1, &indBufObjB);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indBufObjB);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, indBuf.size()*sizeof(unsigned int), &indBuf[0], GL_STATIC_DRAW);

}

void initGround() {

  float G_edge = 2;
  GLfloat g_backgnd_data[] = {
      -G_edge, 0.0f, -G_edge,
        -G_edge,  0.0f, G_edge,
        G_edge, 0.0f, -G_edge,
        -G_edge,  0.0f, G_edge,
        G_edge, 0.0f, -G_edge,
        G_edge, 0.0f, G_edge,
  };

  GLfloat nor_Buf_G[] = { 
      0.0f, 1.0f, 0.0f,
      0.0f, 1.0f, 0.0f,
      0.0f, 1.0f, 0.0f,
      0.0f, 1.0f, 0.0f,
      0.0f, 1.0f, 0.0f,
      0.0f, 1.0f, 0.0f,
  };

   glGenBuffers(1, &posBufObjG);
   glBindBuffer(GL_ARRAY_BUFFER, posBufObjG);
   glBufferData(GL_ARRAY_BUFFER, sizeof(g_backgnd_data), g_backgnd_data, GL_STATIC_DRAW);
	
   glGenBuffers(1, &norBufObjG);
	glBindBuffer(GL_ARRAY_BUFFER, norBufObjG);
	glBufferData(GL_ARRAY_BUFFER, sizeof(nor_Buf_G), nor_Buf_G, GL_STATIC_DRAW);

}

void initCube(std::vector<tinyobj::shape_t>& shape)
{
	// Send the position array to the GPU
	const vector<float> &posBuf = shape[0].mesh.positions;
	glGenBuffers(1, &posBufObjC);
	glBindBuffer(GL_ARRAY_BUFFER, posBufObjC);
	glBufferData(GL_ARRAY_BUFFER, posBuf.size()*sizeof(float), &posBuf[0], GL_STATIC_DRAW);
	
	// Send the normal array to the GPU
	const vector<float> &norBuf = shape[0].mesh.normals;
	glGenBuffers(1, &norBufObjC);
	glBindBuffer(GL_ARRAY_BUFFER, norBufObjC);
	glBufferData(GL_ARRAY_BUFFER, norBuf.size()*sizeof(float), &norBuf[0], GL_STATIC_DRAW);
	
	// Send the index array to the GPU
	const vector<unsigned int> &indBuf = shape[0].mesh.indices;
	glGenBuffers(1, &indBufObjC);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indBufObjC);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, indBuf.size()*sizeof(unsigned int), &indBuf[0], GL_STATIC_DRAW);

	// Unbind the arrays
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	GLSL::checkVersion();
	assert(glGetError() == GL_NO_ERROR);

   //initialize the modeltrans matrix stack
   ModelTrans.useModelViewMatrix();
}

void initGL()
{
	// Set the background color
	glClearColor(0.6f, 0.6f, 0.8f, 1.0f);
	// Enable Z-buffer test
	glEnable(GL_DEPTH_TEST);
    glPointSize(18);
	
    initModel(dog);
    //initBunny(body);
	initCube(cube);
    initGround();
        
	// Unbind the arrays
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	GLSL::checkVersion();
	assert(glGetError() == GL_NO_ERROR);
}


bool installShaders(const string &vShaderName, const string &fShaderName)
{
	GLint rc;
	
	// Create shader handles
	GLuint VS = glCreateShader(GL_VERTEX_SHADER);
	GLuint FS = glCreateShader(GL_FRAGMENT_SHADER);
	
	// Read shader sources
	const char *vshader = GLSL::textFileRead(vShaderName.c_str());
	const char *fshader = GLSL::textFileRead(fShaderName.c_str());
	glShaderSource(VS, 1, &vshader, NULL);
	glShaderSource(FS, 1, &fshader, NULL);
	
	// Compile vertex shader
	glCompileShader(VS);
	GLSL::printError();
	glGetShaderiv(VS, GL_COMPILE_STATUS, &rc);
	GLSL::printShaderInfoLog(VS);
	if(!rc) {
		printf("Error compiling vertex shader %s\n", vShaderName.c_str());
		return false;
	}
	
	// Compile fragment shader
	glCompileShader(FS);
	GLSL::printError();
	glGetShaderiv(FS, GL_COMPILE_STATUS, &rc);
	GLSL::printShaderInfoLog(FS);
	if(!rc) {
		printf("Error compiling fragment shader %s\n", fShaderName.c_str());
		return false;
	}
	
		// Create the program and link
	   ShadeProg = glCreateProgram();
	   glAttachShader(ShadeProg, VS);
	   glAttachShader(ShadeProg, FS);
	   glLinkProgram(ShadeProg);
   
	   GLSL::printError();
	   glGetProgramiv(ShadeProg, GL_LINK_STATUS, &rc);
	   GLSL::printProgramInfoLog(ShadeProg);
	   if(!rc) {
		   printf("Error linking shaders %s and %s\n", vShaderName.c_str(), fShaderName.c_str());
		   return false;
	   }

   /* get handles to attribute data */
    h_aPosition = GLSL::getAttribLocation(ShadeProg, "aPos");
    h_aNormal = GLSL::getAttribLocation(ShadeProg, "aNor");
    h_uProjMatrix = GLSL::getUniformLocation(ShadeProg, "P");
    h_uViewMatrix = GLSL::getUniformLocation(ShadeProg, "uViewMatrix");
    h_uModelMatrix = GLSL::getUniformLocation(ShadeProg, "uModelMatrix");
    h_uLightPos = GLSL::getUniformLocation(ShadeProg, "uLightPos");
    h_uMatAmb = GLSL::getUniformLocation(ShadeProg, "UaColor");
    h_uMatDif = GLSL::getUniformLocation(ShadeProg, "UdColor");
    h_uMatSpec = GLSL::getUniformLocation(ShadeProg, "UsColor");
    h_uMatShine = GLSL::getUniformLocation(ShadeProg, "Ushine");
    h_uShadeM = GLSL::getUniformLocation(ShadeProg, "uShadeModel");
    uNormal = GLSL::getUniformLocation(ShadeProg, "uNormal");
	
	assert(glGetError() == GL_NO_ERROR);
	return true;
}

void enableAndBind(GLuint posBufObj, GLuint norBufObj, 
    GLuint indBufObj, std::vector<tinyobj::shape_t>& shape)
{
    // Enable and bind position array for drawing
    GLSL::enableVertexAttribArray(h_aPosition);
    glBindBuffer(GL_ARRAY_BUFFER, posBufObj);
    glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    // Enable and bind normal array for drawing
    GLSL::enableVertexAttribArray(h_aNormal);
    glBindBuffer(GL_ARRAY_BUFFER, norBufObj);
    glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    // Bind index array for drawing
    nIndices = (int)shape[0].mesh.indices.size();
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indBufObj);

}

void disable()
{
	GLSL::disableVertexAttribArray(h_aPosition);
	GLSL::disableVertexAttribArray(h_aNormal);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
}

void drawGL()
{
	int z = 0;
	// Clear the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	// Use our GLSL program
	glUseProgram(ShadeProg);

    SetProjectionMatrix();
    SetView();

    //setting up the model
	center = vec3(0, 0, -1) + g_trans;
	SetModel(vec3(0, -1.85, 0) + g_trans, 180 + g_angle, 0.2, 1);
    SetMaterial(2);
    //set up light
    glUniform3f(h_uLightPos, g_light.x, g_light.y, g_light.z);
    glUniform1i(h_uShadeM, 1);
    glUniform1i(uNormal, 0);

    //Enable and bind position array for drawing
    enableAndBind(posBufObjB, norBufObjB, indBufObjB, dog);
    //draw the dog
	glDrawElements(GL_TRIANGLES, nIndices, GL_UNSIGNED_INT, 0);
	disable();
	
    //draw the ground
    SetMaterial(4 + material);
   

    glEnableVertexAttribArray(h_aPosition);
    glBindBuffer(GL_ARRAY_BUFFER, posBufObjG);
    glVertexAttribPointer( h_aPosition, 3,  GL_FLOAT, GL_FALSE, 0, (void*)0);
    GLSL::enableVertexAttribArray(h_aNormal);
    glBindBuffer(GL_ARRAY_BUFFER, norBufObjG);
    glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
	zIndex = ((int)(-g_trans.z / 4)) % 10;
	//cout << g_trans.z << " = " << zIndex << endl;
	//cout << wallIndex << ", " << zIndex << "=" <<walls[wallIndex][zIndex] << endl;
	int ztemp;
	while (z > -40) {
		ztemp = (int) -z / 4;
		//cout << "z is " << ztemp << endl;
		if (walls[0][ztemp])
		{
			SetModel(vec3(0, -2, z), 0, 1, 2);
			glDrawArrays(GL_TRIANGLES, 0, 6);
		}
		if (walls[1][ztemp])
		{
			SetModel(vec3(-2, 0, z), -90, 1, 2);
			glDrawArrays(GL_TRIANGLES, 0, 6);
		}

		if (walls[2][ztemp])
		{
			SetModel(vec3(0, 2, z), 180, 1, 2);
			glDrawArrays(GL_TRIANGLES, 0, 6);
		}

		if (walls[3][ztemp])
		{
			SetModel(vec3(2, 0, z), 90, 1, 2);
			glDrawArrays(GL_TRIANGLES, 0, 6);
		}
		z -= 4;
	}

	GLSL::disableVertexAttribArray(h_aPosition);
	GLSL::disableVertexAttribArray(h_aNormal);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	
	// Disable and unbind
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	glUseProgram(0);
	assert(glGetError() == GL_NO_ERROR);
	
}

void window_size_callback(GLFWwindow* window, int w, int h){
	glViewport(0, 0, (GLsizei)w, (GLsizei)h);
	g_width = w;
	g_height = h;
}

void scroll_callback(GLFWwindow* window, double x, double y)
{
    double xOff = x * 0.04;
    double yOff = y * 0.04;

    if (phi + yOff <= 3.0 / 2.0 && phi + yOff >= - (3.0 / 2.0))
    {
        phi += yOff;
    }
    theta += xOff;
    center.x = eye.x + cos(phi) * cos(theta);
    center.y = eye.y + sin(phi);
    center.z = eye.z + cos(phi) * cos(pi / 2 - theta);
}

void generateMap()
{
	for (int row = 0; row < 4; row++)
	{
		for (int col = 0; col < 10; col++)
		{
			walls[row][col] = 1;
		}
	}

	for (int col = 0; col < 10; col++)
	{
		int random = rand() % 4;
		walls[random][col] = 0;
	}
	walls[wallIndex][0] = 1;
}

void key_callback(GLFWwindow* window, int key, int scancode, int action, int mods)
{
    glm::vec3 view = glm::normalize(center - eye);

	if (key == GLFW_KEY_T && action == GLFW_PRESS)
	{
		gameStart = true;
	}
	if (key == GLFW_KEY_R && action == GLFW_PRESS)
	{
		gameStart = false;
		wallIndex = 0;
		zIndex = 0;
		generateMap();
		eye = vec3(0, 0, 6);
		g_trans = vec3(0);
		falling = false;
		jumpTime = 0;
		leftRotate = 0;
		rightRotate = 0;
		time = 0;
	}
}

int main(int argc, char **argv)
{

// Initialise GLFW
    if( !glfwInit() )
    {
        fprintf( stderr, "Failed to initialize GLFW\n" );
        return -1;
    }

    glfwWindowHint(GLFW_SAMPLES, 4);
   glfwWindowHint(GLFW_RESIZABLE,GL_FALSE);
   glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 2);
   glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 1);

    // Open a window and create its OpenGL context
   g_width = 1024;
   g_height = 768;
   window = glfwCreateWindow( g_width, g_height, "Final Project", NULL, NULL);
   if( window == NULL ){
      fprintf( stderr, "Failed to open GLFW window. If you have an Intel GPU, they are not 3.3 compatible. Try the 2.1 version of the tutorials.\n" );
        glfwTerminate();
        return -1;
    }
    glfwMakeContextCurrent(window);
    glfwSetKeyCallback(window, key_callback);
    //glfwSetScrollCallback(window, scroll_callback);
    glfwSetWindowSizeCallback(window, window_size_callback);
// Initialize GLEW
   if (glewInit() != GLEW_OK) {
      fprintf(stderr, "Failed to initialize GLEW\n");
      return -1;
   }

	// Ensure we can capture the escape key being pressed below
	glfwSetInputMode(window, GLFW_STICKY_KEYS, GL_TRUE);

    glEnable (GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	loadShapes("dog.obj", dog);
	loadShapes("cube.obj", cube);

	initGL();
	installShaders("vert.glsl", "frag.glsl");

	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	generateMap();
    do{
      drawGL();
	  time += 0.1;
	  g_angle = 0;
	  //handle key clicks
	  glm::vec3 view = glm::normalize(center - eye);
	  if (glfwGetKey(window, GLFW_KEY_LEFT) == GLFW_PRESS) {
		  if (g_trans.x > -2)
		  {
			glm::vec3 strafe = glm::cross(view, vec3(0, 1, 0));
			eye.x -= strafe.x * SPEED;
			eye.y -= strafe.y * SPEED;
			eye.z -= strafe.z * SPEED;
			g_trans.x -= strafe.x * SPEED;
			g_trans.y -= strafe.y * SPEED;
			g_trans.z -= strafe.z * SPEED;
			g_angle = 30;
		  }
	  }
	  if (glfwGetKey(window, GLFW_KEY_RIGHT) == GLFW_PRESS) {
		  if (g_trans.x < 2)
		  {
			glm::vec3 strafe = glm::cross(view, vec3(0, 1, 0));
			eye.x += strafe.x * SPEED;
			eye.y += strafe.y * SPEED;
			eye.z += strafe.z * SPEED;
			g_trans.x += strafe.x * SPEED;
			g_trans.y += strafe.y * SPEED;
			g_trans.z += strafe.z * SPEED;
			g_angle = -30;
		  }
	  }
	if (glfwGetKey(window, GLFW_KEY_SPACE) == GLFW_PRESS)
	{
		//cout << g_trans.y << " " << jumpTime << "\n";
		if (g_trans.y < maxHeight && jumpTime < 8)
		{
			eye.y += SPEED;
			g_trans.y += SPEED;
			jumpReady = false;
		}
	}
	if (gameStart && !falling)
	{
		eye.z -= addSpeed + SPEED;
		g_trans.z -= addSpeed + SPEED;
		jumpTime += 0.1;
	}
	if (g_trans.y > 0 && !falling)
	{
		eye.y -= 0.5 * SPEED;
		g_trans.y -= 0.5 * SPEED;
	}
	if (g_trans.y <= 0)
	{
		if (walls[wallIndex][zIndex] == 0)
		{
			g_trans.y -= 0.1;
			if (!falling)
			{
				cout << "You final score is: " << time << "!" << endl;
				if (time < 100)
					cout << "Rank: Bronze" << endl;
				else if (time < 300)
					cout << "Rank: Silver" << endl;
				else if (time < 500)
					cout << "Rank: Gold" << endl;
				else if (time < 700)
					cout << "Rank: Platinum!" << endl;
				else
					cout << "Rank: Diamond Playaaa!" << endl;
				
				cout << endl;
			}
			falling = true;
		}
		else
		{
			g_trans.y = 0;
			eye.y = 0;
			jumpTime = 0;
		}
	}
	if (g_trans.x <= -2 && !falling)
	{
		g_trans.x = 0;
		center.x = 0;
		eye.x = 0;
		leftRotate += LEFT;
		wallIndex++;
		if (wallIndex > 3)
			wallIndex = 0;
	}
	if (g_trans.x >= 2 && !falling)
	{
		g_trans.x = 0;
		center.x = 0;
		eye.x = 0;
		rightRotate += RIGHT;
		wallIndex--;
		if (wallIndex < 0)
			wallIndex = 3;
	}
	if (g_trans.z <= -40 && !falling)
	{
		g_trans.z = 0;
		eye.z = 6;
		wallIndex = 0;
		zIndex = 0;
		generateMap();
		jumpTime = 0;
		leftRotate = 0;
		rightRotate = 0;
		addSpeed += 0.002;
		material++;
		if (material == 2)
			material = 0;
	}
        // Swap buffers
      glfwSwapBuffers(window);
      glfwPollEvents();

    } // Check if the ESC key was pressed or the window was closed
   while( glfwGetKey(window, GLFW_KEY_ESCAPE ) != GLFW_PRESS &&
         glfwWindowShouldClose(window) == 0 );

    // Close OpenGL window and terminate GLFW
    glfwTerminate();

	return 0;
}
